@@ -1,3 +1,24 @@
+##############################################################################
+#    Copyright (c) 2012-2016 Russell V. Lenth                                #
+#                                                                            #
+#    This file is part of the lsmeans package for R (*lsmeans*)              #
+#                                                                            #
+#    *lsmeans* is free software: you can redistribute it and/or modify       #
+#    it under the terms of the GNU General Public License as published by    #
+#    the Free Software Foundation, either version 2 of the License, or       #
+#    (at your option) any later version.                                     #
+#                                                                            #
+#    *lsmeans* is distributed in the hope that it will be useful,            #
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
+#    GNU General Public License for more details.                            #
+#                                                                            #
+#    You should have received a copy of the GNU General Public License       #
+#    along with R and *lsmeans*.  If not, see                                #
+#    <https://www.r-project.org/Licenses/> and/or                            #
+#    <http://www.gnu.org/licenses/>.                                         #
+##############################################################################
+
 ### Helper functions for lsmeans
 ### Here we have 'recover.data' and 'lsm.basis' methods
 ### For models that this package supports.
@@ -71,36 +92,58 @@ lsm.basis.default = function(object, trms, xlev, grid, ...) {
 # For model objects, call this with the object's call and its terms component
 # Late addition: if data is non-null, use it in place of recovered data
 # Later addition: na.action arg req'd - vector of row indexes removed due to NAs
+#    na.action is ignored when data is non-NULL
 recover.data.call = function(object, trms, na.action, data = NULL, params = NULL, ...) {
     fcall = object # because I'm easily confused
     vars = setdiff(.all.vars(trms), params)
-    if (length(vars) == 0)
-        return("Model must have at least one predictor")
     tbl = data
+    if (length(vars) == 0) {
+        tbl = data.frame(c(1,1))
+        vars = names(tbl) = 1
+    }
     if (is.null(tbl)) {
         m = match(c("formula", "data", "subset", "weights"), names(fcall), 0L)
         fcall = fcall[c(1L, m)]
+        
+        # check to see if there are any function calls to worry about
+        # [e.g., subset = sample(1:n, 50) will give us a different subset than model used]
+        mm = match(c("data", "subset"), names(fcall), 0L)
+        if(any(mm > 0)) {
+            fcns = unlist(lapply(fcall[mm], 
+                        function(x) setdiff(all.names(x), c("::",":::","[[","]]",all.vars(x)))))
+            if(max(nchar(c("", fcns))) > 1)
+                warning("Function call in data or subset: ref.grid/lsmeans results may be inconsistent",
+                        call. = FALSE)
+        }
+        
         fcall$drop.unused.levels = TRUE
         fcall[[1L]] = as.name("model.frame")
         fcall$xlev = NULL # we'll ignore xlev
-        fcall$na.action = na.omit
-        ### I once had a reason to put one var on the left, but don't remember why
-        ### Now it messes up if there's a '$' in that term
-        #         if (length(vars) > 1) 
-        #             form = reformulate(vars[-1], response = vars[1])
-        #         else
+        
+        if(!is.numeric(na.action))   ### In case na.action is not a vector of indices
+            na.action = NULL
+        
+        # If we have an explicit list of cases to exclude, let everything through now
+        if (!is.null(na.action))
+            fcall$na.action = na.pass
+        else  # exclude incomplete cases
+            fcall$na.action = na.omit
         form = reformulate(vars)
         fcall$formula = update(trms, form)
         env = environment(trms)
         if (is.null(env)) 
             env = parent.frame()
         tbl = eval(fcall, env, parent.frame())
+        
+        # Now we can drop na.action's rows
         if (!is.null(na.action))
             tbl = tbl[-(na.action),  , drop=FALSE]
     }
     
-    else
-        fcall$data = tbl[complete.cases(data), , drop=FALSE]
+    else {
+        tbl = tbl[, vars, drop = FALSE] # consider only the variables actually needed
+        tbl = tbl[complete.cases(tbl), , drop=FALSE]
+    }
     
     attr(tbl, "call") = object # the original call
     attr(tbl, "terms") = trms
@@ -118,10 +161,12 @@ recover.data.lm = function(object, ...) {
 }
 
 lsm.basis.lm = function(object, trms, xlev, grid, ...) {
-    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
-    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
     # coef() works right for lm but coef.aov tosses out NAs
-    bhat = as.numeric(object$coefficients) 
+    bhat = object$coefficients
+    nm = if(is.null(names(bhat))) row.names(bhat) else names(bhat)
+    m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
+    X = model.matrix(trms, m, contrasts.arg = object$contrasts)[, nm, drop = FALSE]
+    bhat = as.numeric(bhat) 
     # stretches it out if multivariate - see mlm method
     V = .my.vcov(object, ...)
     
@@ -173,40 +218,64 @@ recover.data.merMod = function(object, ...) {
                  attr(object@frame, "na.action"), ...)
 }
 
-lsm.basis.merMod = function(object, trms, xlev, grid, vcov., ...) {
+lsm.basis.merMod = function(object, trms, xlev, grid, vcov., 
+                            mode = get.lsm.option("lmer.df"), lmer.df, ...) {
     if (missing(vcov.))
-        V = as.matrix(vcov(object))
+        V = as.matrix(vcov(object, correlation = FALSE))
     else
         V = as.matrix(.my.vcov(object, vcov.))
     dfargs = misc = list()
+    
     if (lme4::isLMM(object)) {
-        pbdis = .lsm.is.true("disable.pbkrtest")
-        Nlim = get.lsm.option("pbkrtest.limit")
-        objN = lme4::getME(object, "N")
-        toobig = objN > Nlim
-        if (!pbdis && !toobig && requireNamespace("pbkrtest") && missing(vcov.)) {
-            dfargs = list(unadjV = V, 
-                adjV = pbkrtest::vcovAdj.lmerMod(object, 0))
-            V = as.matrix(dfargs$adjV)
-            tst = try(pbkrtest::Lb_ddf)
-            if(class(tst) != "try-error")
-                dffun = function(k, dfargs) pbkrtest::Lb_ddf (k, dfargs$unadjV, dfargs$adjV)
+        # Allow user to specify mode as 'lmer.df'
+        if (!missing(lmer.df))
+            mode = lmer.df
+        mode = match.arg(tolower(mode), c("satterthwaite", "kenward-roger", "asymptotic"))
+        
+        if (mode == "satterthwaite") {
+            if (requireNamespace("lmerTest")) {
+                dfargs = list(object = object)
+                dffun = function(k, dfargs) 
+                    suppressMessages(lmerTest::calcSatterth(dfargs$object, k)$denom)
+            }
             else {
-                dffun = function(k, dfargs) NA
-                warning("To obtain d.f., install 'pbkrtest' version 0.4-1 or later")
+                message("Install package 'lmerTest' to obtain Satterthwaite degrees of freedom")
+                mode = "asymptotic"
             }
         }
-        else {
-            if(!pbdis && !("pbkrtest" %in% row.names(installed.packages())))
-                message("Install package 'pbkrtest' to obtain bias corrections and degrees of freedom")
-            else if(toobig)
-                message("Note: Adjusted covariance and degrees-of-freedom calculations have been\n",
-                        "disabled because the number of observations exceeds ", Nlim, ".\n",
-                        "Standard errors and tests may be more biased than if they were adjusted.\n",
-                        "To enable adjustments, set lsm.options(pbkrtest.limit = ", objN, ") or larger,\n",
-                        "but be warned that this may result in large computation time and memory use.")
+        else if (mode == "kenward-roger") {
+            pbdis = .lsm.is.true("disable.pbkrtest")
+            Nlim = get.lsm.option("pbkrtest.limit")
+            objN = lme4::getME(object, "N")
+            toobig = objN > Nlim
+            if (!pbdis && !toobig && requireNamespace("pbkrtest") && missing(vcov.)) {
+                dfargs = list(unadjV = V, 
+                              adjV = pbkrtest::vcovAdj.lmerMod(object, 0))
+                V = as.matrix(dfargs$adjV)
+                tst = try(pbkrtest::Lb_ddf)
+                if(class(tst) != "try-error")
+                    dffun = function(k, dfargs) pbkrtest::Lb_ddf (k, dfargs$unadjV, dfargs$adjV)
+                else {
+                    mode = "asymptotic"
+                    warning("To obtain d.f., install 'pbkrtest' version 0.4-1 or later")
+                }
+            }
+            else {
+                if(!pbdis && !("pbkrtest" %in% row.names(installed.packages())))
+                    message("Install package 'pbkrtest' to obtain bias corrections and degrees of freedom")
+                else if(toobig)
+                    message("Note: Adjusted covariance and degrees-of-freedom calculations have been\n",
+                            "disabled because the number of observations exceeds ", Nlim, ".\n",
+                            "Standard errors and tests may be more biased than if they were adjusted.\n",
+                            "To enable adjustments, set lsm.options(pbkrtest.limit = ", objN, ") or larger,\n",
+                            "but be warned that this may result in large computation time and memory use.")
+                mode = "asymptotic"
+            }
+        }
+        if (mode == "asymptotic") {
             dffun = function(k, dfargs) NA
         }
+        misc$initMesg = paste("Degrees-of-freedom method:", mode)
     }
     else if (lme4::isGLMM(object)) {
         dffun = function(k, dfargs) NA
@@ -240,46 +309,47 @@ lsm.basis.merMod = function(object, trms, xlev, grid, vcov., ...) {
 
 
 
+# lme4.0 support removed because CRAN really doesn't want it supported 
 #--------------------------------------------------------------
-### mer objects (from old lme4 version, now lme4.0)
-### re-enabled; CRAN check now seems to work with multiple add'l repositories
-
-recover.data.mer = function(object, ...) {
-    if(!lme4.0::isLMM(object) && !lme4.0::isGLMM(object)) 
-        return("Can't handle a nonlinear mixed model")
-    fcall = object@call
-    recover.data(fcall, delete.response(terms(object)), 
-                 attr(object@frame, "na.action"), ...)
-}
-
-# Does NOT support pbkrtest capabilities. Uses asymptotic methods
-lsm.basis.mer = function(object, trms, xlev, grid, ...) {
-    V = as.matrix(.my.vcov(object, ...))
-    dfargs = misc = list()
-    if (lme4.0::isLMM(object)) {
-        dffun = function(k, dfargs) NA        
-    }
-    else if (lme4.0::isGLMM(object)) {
-        dffun = function(k, dfargs) NA
-        # need to work harder as there is no 'family' method
-        cfam = object@call$family
-        if (is.name(cfam))
-            fam = eval(cfam)()
-        else
-            fam = eval(cfam)
-        misc = .std.link.labels(fam, misc)
-    }
-    else 
-        stop("Can't handle a nonlinear mixed model")
-    
-    contrasts = attr(object@X, "contrasts")
-    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
-    X = model.matrix(trms, m, contrasts.arg = contrasts)
-    bhat = lme4.0::fixef(object)
-    nbasis=estimability::all.estble
-    
-    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
-}
+# ### mer objects (from old lme4 version, now lme4.0)
+# ### re-enabled; CRAN check now seems to work with multiple add'l repositories
+# 
+# recover.data.mer = function(object, ...) {
+#     if(!lme4.0::isLMM(object) && !lme4.0::isGLMM(object)) 
+#         return("Can't handle a nonlinear mixed model")
+#     fcall = object@call
+#     recover.data(fcall, delete.response(terms(object)), 
+#                  attr(object@frame, "na.action"), ...)
+# }
+# 
+# # Does NOT support pbkrtest capabilities. Uses asymptotic methods
+# lsm.basis.mer = function(object, trms, xlev, grid, ...) {
+#     V = as.matrix(.my.vcov(object, ...))
+#     dfargs = misc = list()
+#     if (lme4.0::isLMM(object)) {
+#         dffun = function(k, dfargs) NA        
+#     }
+#     else if (lme4.0::isGLMM(object)) {
+#         dffun = function(k, dfargs) NA
+#         # need to work harder as there is no 'family' method
+#         cfam = object@call$family
+#         if (is.name(cfam))
+#             fam = eval(cfam)()
+#         else
+#             fam = eval(cfam)
+#         misc = .std.link.labels(fam, misc)
+#     }
+#     else 
+#         stop("Can't handle a nonlinear mixed model")
+#     
+#     contrasts = attr(object@X, "contrasts")
+#     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
+#     X = model.matrix(trms, m, contrasts.arg = contrasts)
+#     bhat = lme4.0::fixef(object)
+#     nbasis=estimability::all.estble
+#     
+#     list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
+# }
 
 
 
@@ -290,18 +360,16 @@ recover.data.lme = function(object, data, ...) {
     fcall = object$call
     if (!is.null(fcall$weights))
         fcall$weights = nlme::varWeights(object$modelStruct)
-    if(is.null(data)) # lme objects actually have the data, so use it!
-        data = object$data
     recover.data(fcall, delete.response(terms(object)), object$na.action, data = data, ...)
 }
 
-lsm.basis.lme = function(object, trms, xlev, grid, adjustSigma = TRUE, ...) {
+lsm.basis.lme = function(object, trms, xlev, grid, sigmaAdjust = TRUE, ...) {
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
     bhat = nlme::fixef(object)
     V = .my.vcov(object, ...)
-    if (adjustSigma && object$method == "ML") 
+    if (sigmaAdjust && object$method == "ML") 
         V = V * object$dims$N / (object$dims$N - nrow(V))
     misc = list()
     if (!is.null(object$family)) {
@@ -351,7 +419,7 @@ lsm.basis.gls = function(object, trms, xlev, grid, ...) {
 recover.data.polr = recover.data.lm
 
 lsm.basis.polr = function(object, trms, xlev, grid, 
-                          mode = c("latent", "linear.predictor", "cum.prob", "prob", "mean.class"), 
+                          mode = c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"), 
                           rescale = c(0,1), ...) {
     mode = match.arg(mode)
     contrasts = object$contrasts
@@ -658,8 +726,8 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
 #     recover.data(fcall, delete.response(terms(object$lme)), object$lme$na.action, ...)
 # }
 # 
-# lsm.basis.gamm = function (object, trms, xlev, grid, adjustSigma = TRUE, ...) {
-#     lsm.basis(object$lme, trms, xlev, grid, adjustSigma, ...)
+# lsm.basis.gamm = function (object, trms, xlev, grid, sigmaAdjust = TRUE, ...) {
+#     lsm.basis(object$lme, trms, xlev, grid, sigmaAdjust, ...)
 #     # Doesn't work because needs the matrices in object$lme$data
 # }
 
@@ -671,7 +739,8 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
 ### ----- Auxiliary routines -------------------------
 # Provide for vcov. argument in ref.grid call, which could be a function or a matrix
 
-.my.vcov = function(object, vcov. = stats::vcov, ...) {
+.my.vcov = function(object, vcov. = function(object) 
+                            stats::vcov(object, complete = FALSE), ...) {
     if (is.function(vcov.))
         vcov. = vcov.(object)
     else if (!is.matrix(vcov.))
@@ -684,6 +753,8 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
 .std.link.labels = function(fam, misc) {
     if (is.null(fam))
         return(misc)
+    if (fam$link == "identity")
+        return(misc)
     misc$tran = fam$link
     misc$inv.lbl = "response"
     if (length(grep("binomial", fam$family)) == 1)
@@ -696,6 +767,8 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
 ## Alternative to all.vars, but keeps vars like foo$x and foo[[1]] as-is
 ##   Passes ... to all.vars
 .all.vars = function(expr, retain = c("\\$", "\\[\\[", "\\]\\]"), ...) {
+    if (is.null(expr))
+        return(character(0))
     if (!inherits(expr, "formula")) {
         expr = try(eval(expr), silent = TRUE)
         if(inherits(expr, "try-error")) {
@@ -710,6 +783,7 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
     retain = gsub("\\\\", "", retain)
     for (i in seq_along(retain))
         vars = gsub(repl[i], retain[i], vars)
+    if(length(vars) == 0) vars = "1"   # no vars ---> intercept
     vars
 }
 
