@@ -1,3 +1,24 @@
+##############################################################################
+#    Copyright (c) 2012-2016 Russell V. Lenth                                #
+#                                                                            #
+#    This file is part of the lsmeans package for R (*lsmeans*)              #
+#                                                                            #
+#    *lsmeans* is free software: you can redistribute it and/or modify       #
+#    it under the terms of the GNU General Public License as published by    #
+#    the Free Software Foundation, either version 2 of the License, or       #
+#    (at your option) any later version.                                     #
+#                                                                            #
+#    *lsmeans* is distributed in the hope that it will be useful,            #
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
+#    GNU General Public License for more details.                            #
+#                                                                            #
+#    You should have received a copy of the GNU General Public License       #
+#    along with R and *lsmeans*.  If not, see                                #
+#    <https://www.r-project.org/Licenses/> and/or                            #
+#    <http://www.gnu.org/licenses/>.                                         #
+##############################################################################
+
 # Reference grid code
 
 
@@ -8,7 +29,16 @@
 #     FALSE - same as function(x) sort(unique(x))
 
 ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs, 
-                     options = get.lsm.option("ref.grid"), data, type, ...) {
+                     options = get.lsm.option("ref.grid"), data, df, type, 
+                     transform = c("none", "response", "mu", "unlink", "log"), 
+                     nesting, ...) 
+{
+    transform = match.arg(transform)
+    if (!missing(df)) {
+        if(is.null(options)) options = list()
+        options$df = df
+    }
+    
     # recover the data
     if (missing(data)) {
         data = try(recover.data (object, data = NULL, ...))
@@ -24,10 +54,8 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
     
     trms = attr(data, "terms")
     
-    # find out if any variables are coerced to factors
-    ### OLD VERSION: anm = all.names(attr(data, "terms"))    
-    ###              coerced = anm[1 + grep("factor|ordered", anm)]
-    coerced = .find.coerced(trms, data)
+    # find out if any variables are coerced to factors or vice versa
+    coerced = .find.coerced(trms, data) # now list with members 'factors' and 'covariates'
     
     # convenience function
     sort.unique = function(x) sort(unique(x))
@@ -66,7 +94,7 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
     cr = function(x, nm) {
         if (is.function(cov.reduce))
             cov.reduce(x)
-        else if (!is.null(cov.reduce[[nm]]))
+        else if (hasName(cov.reduce, nm))
             cov.reduce[[nm]](x)
         else
             mean(x)
@@ -87,16 +115,16 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
         x = data[[nm]]
         
         # Save the original levels of factors, no matter what
-        if (is.factor(x))
+        if (is.factor(x) && !(nm %in% coerced$covariates))
             xlev[[nm]] = levels(factor(x))
             # (applying factor drops any unused levels)
     
         # Now go thru and find reference levels...
-        # mentioned in 'at' list but not coerced
-        if (!(nm %in% coerced) && !missing(at) && !is.null(at[[nm]]))
+        # mentioned in 'at' list but not coerced factor
+        if (!(nm %in% coerced$factors) && !missing(at) && (hasName(at, nm)))
             ref.levels[[nm]] = at[[nm]]
         # factors not in 'at'
-        else if (is.factor(x))
+        else if (is.factor(x) && !(nm %in% coerced$covariates))
             ref.levels[[nm]] = levels(factor(x))
         else if (is.character(x))
             ref.levels[[nm]] = sort.unique(x)
@@ -112,12 +140,12 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
         else {
             # single numeric pred but coerced to a factor - use unique values
             # even if in 'at' list. We'll fix this up later
-            if (nm %in% coerced)            
+            if (nm %in% coerced$factors)            
                 ref.levels[[nm]] = sort.unique(x)
             
             # Ordinary covariates - summarize
             else 
-                ref.levels[[nm]] = cr(x, nm)
+                ref.levels[[nm]] = cr(as.numeric(x), nm)
         }
     }
     
@@ -141,12 +169,31 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
     
     misc = basis$misc
     
-    form = attr(data, "call")$formula
-    if (is.null(misc$tran) && (length(form) > 2)) { # No link fcn, but response may be transformed
-        lhs = form[-3] ####form[[2]]
-        tran = setdiff(.all.vars(lhs, functions = TRUE), c(.all.vars(lhs), "~", "cbind"))
-        if(length(tran) == 1) {
-            misc$tran = tran
+    ### Figure out if there is a response transformation...
+    # next stmt assumes that model formula is 1st argument (2nd element) in call.
+    # if not, we probably get an error or something that isn't a formula
+    # and it is silently ignored
+    lhs = try(eval(attr(data, "call")[[2]][-3]), silent = TRUE)
+    if (inherits(lhs, "formula")) { # response may be transformed
+        tran = setdiff(.all.vars(lhs, functions = TRUE), c(.all.vars(lhs), "~", "cbind", "+", "-", "*", "/", "^", "%%", "%/%"))
+        if(length(tran) > 0) {
+            tran = paste(tran, collapse = ".")  
+            # length > 1: Almost certainly unsupported, but facilitates a more informative error message
+            
+            # Look for a multiplier, e.g. 2*sqrt(y)
+            tst = strsplit(strsplit(as.character(lhs[2]), "\\(")[[1]][1], "\\*")[[1]]
+            if(length(tst) > 1) {
+                mul = suppressWarnings(as.numeric(tst[1]))
+                if(!is.na(mul))
+                    misc$tran.mult = mul
+                tran = gsub("\\*\\.", "", tran)
+            }
+            if (tran == "linkfun")
+                tran = as.list(environment(trms))
+            if(is.null(misc$tran))
+                misc$tran = tran
+            else
+                misc$tran2 = tran
             misc$inv.lbl = "response"
         }
     }
@@ -185,9 +232,9 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
 # So we now need to subset the rows of the grid and linfct based on 'at'
 
     problems = if (!missing(at)) 
-        intersect(c(multresp, coerced), names(at)) 
+        intersect(c(multresp, coerced$factors), names(at)) 
     else character(0)
-    if (length(problems > 0)) {
+    if (length(problems) > 0) {
         incl.flags = rep(TRUE, nrow(grid))
         for (nm in problems) {
             if (is.numeric(ref.levels[[nm]])) {
@@ -218,9 +265,9 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
     }
 
     ### --- Determine weights for each grid point --- (added ver.2.11), updated ver.2.14 to include weights
-    if (is.null(data[["(weights)"]]))
+    if (!hasName(data, "(weights)"))
         data[["(weights)"]] = 1
-    nms = union(names(xlev), coerced) # only factors, no covariates or mult.resp
+    nms = union(names(xlev), coerced$factors) # only factors, no covariates or mult.resp
     # originally, I used 'plyr::count', but there are probs when there is a 'freq' variable
     id = plyr::id(data[, nms, drop = FALSE], drop = TRUE)
     uid = !duplicated(id)
@@ -232,6 +279,12 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
     for (i in seq_along(key))
         wgt[tgt == key[i]] = sum(data[["(weights)"]][id==i])
     grid[[".wgt."]] = wgt
+    
+    model.info = list(call = attr(data,"call"), terms = trms, xlev = xlev)
+    # Detect any nesting structures
+    nst = .find_nests(grid, trms, ref.levels)
+    if (length(nst) > 0)
+        model.info$nesting = nst
 
     misc$ylevs = NULL # No longer needed
     misc$estName = "prediction"
@@ -247,7 +300,7 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
         post.beta = matrix(NA)
     
     result = new("ref.grid",
-         model.info = list(call = attr(data,"call"), terms = trms, xlev = xlev),
+         model.info = model.info,
          roles = list(predictors = attr(data, "predictors"), 
                       responses = attr(data, "responses"), 
                       multresp = multresp),
@@ -260,6 +313,9 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
         if (is.null(options)) options = list()
         options$predict.type = type
     }
+    
+    if (!missing(nesting))
+        result@model.info$nesting = .parse_nest(nesting)
 
     if(!is.null(options)) {
         options$object = result
@@ -270,22 +326,38 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
         if (is.character(hook))
             hook = get(hook)
         result@misc$postGridHook = NULL
-        hook(result)
+        result = hook(result)
     }
-    else
-        result
+    if(transform != "none")
+        result = regrid(result, transform = transform)
+    
+    .save.ref.grid(result)
+    result
 }
 
 
 #### End of ref.grid ------------------------------------------
 
+# local utility to identify ref.grid that is not an lsmobj
+.is.true.ref.grid = function(object) {
+    is(object, "ref.grid") && !is(object, "lsmobj")
+}
+
+# local utility to save each newly constructed ref.grid, if enabled
+# Goes into global environment unless .Last.ref.grid is found further up
+.save.ref.grid = function(object) {
+    if(get.lsm.option("save.ref.grid") && .is.true.ref.grid(object))
+        assign(".Last.ref.grid", object, inherits = TRUE)
+}
 
 
 
 # This function figures out which covariates in a model 
-# have been coerced to factors. Does NOT rely on the names of
-# functions like 'factor' or 'interaction' as we use actual results
+# have been coerced to factors. And also which factors have been coerced
+# to be covariates
 .find.coerced = function(trms, data) {
+    if (ncol(data) == 0) 
+        return(list(factors = integer(0), covariates = integer(0)))
     isfac = sapply(data, function(x) inherits(x, "factor"))
     
     # Character vectors of factors and covariates in the data...
@@ -297,19 +369,25 @@ ref.grid <- function(object, at, cov.reduce = mean, mult.name, mult.levs,
     isfac = sapply(M, function(x) inherits(x, "factor"))
     
     # Character vector of terms in the model frame that are factors ...
-    facs.m = names(M)[isfac]
+    facs.m = names(M)[as.logical(isfac)]
+    covs.m = setdiff(names(M), facs.m)
     
     # Exclude the terms that are already factors
     # What's left will be things like "factor(dose)", "interact(dose,treat)", etc
-    cterms = setdiff(facs.m, facs.d)
+    cfac = setdiff(facs.m, facs.d)
+    if(length(cfac) != 0) {
+        cvars = lapply(cfac, function(x) .all.vars(stats::reformulate(x))) # Strip off the function calls
+        cfac = intersect(unique(unlist(cvars)), covs.d) # Exclude any variables that are already factors
+    }
     
-    if(length(cterms) == 0) 
-        return(cterms)
-    # (else) Strip off the function calls
-    cvars = lapply(cterms, function(x) .all.vars(reformulate(x)))
+    # Do same with covariates
+    ccov = setdiff(covs.m, covs.d)
+    if(length(ccov) > 0) {
+        cvars = lapply(ccov, function(x) .all.vars(stats::reformulate(x)))
+        ccov = intersect(unique(unlist(cvars)), facs.d)
+    }
     
-    # Exclude any variables that are already factors
-    intersect(unique(unlist(cvars)), covs.d)
+    list(factors = cfac, covariates = ccov)
 }
 
 # calculate the offset for the given grid
@@ -332,13 +410,19 @@ str.ref.grid <- function(object, ...) {
         if (is.null(x)) cat("(predicted by other variables)")
         else cat(paste(format(x, digits = 5, justify = "none"), collapse=", "))
     }
-    #cat("responses: ")
-    #showlevs(object@roles$responses)
+    showtran = function(tran, label) { # internal convenience fcn
+        if (is.list(tran)) 
+            tran = ifelse(is.null(tran$name), "custom", tran$name)
+        if (!is.null(mul <- object@misc$tran.mult))
+            tran = paste0(mul, "*", tran)
+        cat(paste(label, dQuote(tran), "\n"))
+        
+    }
     levs = object@levels
     cat(paste("'", class(object)[1], "' object with variables:\n", sep=""))
     for (nm in union(object@roles$predictors, union(object@roles$multresp, object@roles$responses))) {
         cat(paste("    ", nm, " = ", sep = ""))
-        if (nm %in% names(object@matlevs)) {
+        if (hasName(object@matlevs, nm)) {
             if (nm %in% object@roles$responses)
                 cat("multivariate response with means: ")
             else
@@ -358,9 +442,15 @@ str.ref.grid <- function(object, ...) {
             showlevs(levs[[nm]])
         cat("\n")
     }
+    if(!is.null(object@model.info$nesting)) {
+        cat("Nesting structure:  ")
+        cat(.fmt.nest(object@model.info$nesting))
+        cat("\n")
+    }
     if(!is.null(tran <- object@misc$tran)) {
-        if (is.list(tran)) tran = "custom - see slot(, \"misc\")$tran"
-        cat(paste("Transformation:", dQuote(tran), "\n"))
+        showtran(tran, "Transformation:")
+        if (!is.null(tran2 <- object@misc$tran2))
+            showtran(tran2, "Additional response transformation:")
     }
 }
 
@@ -391,9 +481,11 @@ vcov.ref.grid = function(object, ...) {
 # Method to alter contents of misc slot
 update.ref.grid = function(object, ..., silent = FALSE) {
     args = list(...)
-    valid.choices = c("adjust","alpha","avgd.over","by.vars","df",
+    valid.misc = c("adjust","alpha","avgd.over","by.vars","delta","df",
         "initMesg","estName","estType","famSize","infer","inv.lbl",
-        "level","methdesc","predict.type","pri.vars","tran")
+        "level","methdesc","nesting","null","predict.type","pri.vars","side","tran","tran.mult","tran2")
+    valid.slots = slotNames(object)
+    valid.choices = union(valid.misc, valid.slots)
     misc = object@misc
     for (nm in names(args)) {
         fullname = try(match.arg(nm, valid.choices), silent=TRUE)
@@ -403,15 +495,22 @@ update.ref.grid = function(object, ..., silent = FALSE) {
                     paste(valid.choices, collapse=", "))
         }
         else {
-            if (fullname == "by.vars") {
-                allvars = union(misc$pri.vars, misc$by.vars)
-                misc$pri.vars = setdiff(allvars, args[[nm]])
+            if (fullname %in% valid.slots)
+                slot(object, fullname) = args[[nm]]
+            else {
+                if (fullname == "by.vars") {
+                    allvars = union(misc$pri.vars, misc$by.vars)
+                    misc$pri.vars = setdiff(allvars, args[[nm]])
+                }
+                if (fullname == "pri.vars") {
+                    allvars = union(misc$pri.vars, misc$by.vars)
+                    misc$by.vars = setdiff(allvars, args[[nm]])
+                }
+                if (fullname == "nesting") # special case - I keep nesting in model.info
+                    object@model.info$nesting = args[[nm]]
+                else
+                    misc[[fullname]] = args[[nm]]
             }
-            if (fullname == "pri.vars") {
-                allvars = union(misc$pri.vars, misc$by.vars)
-                misc$by.vars = setdiff(allvars, args[[nm]])
-            }
-            misc[[fullname]] = args[[nm]]
         }
     }
     object@misc = misc
@@ -426,28 +525,33 @@ lsm.options = function(...) {
     for (nm in names(newopts))
         opts[[nm]] = newopts[[nm]]
     options(lsmeans = opts)
-    invisible(opts)
+    if (length(newopts) > 0)
+        invisible(opts)
+    else
+        opts
 }
 
 # equivalent of getOption()
-get.lsm.option = function(x, default = lsmeans::defaults[[x]]) {
+get.lsm.option = function(x, default = defaults.lsm[[x]]) {
     opts = getOption("lsmeans", list())
-    if(is.null(default) || x %in% names(opts))
+    if(is.null(default) || hasName(opts, x))
         opts[[x]]
     else 
         default
 }
 
 ### Exported defaults for certain options
-defaults = list(
+defaults.lsm = list(
     estble.tol = 1e-8,        # tolerance for estimability checks
+    lmer.df = "satterth",     # Use Satterthwaite method for df
     disable.pbkrtest = FALSE, # whether to bypass pbkrtest routines for lmerMod
-    pbkrtest.limit = 3000     # limit on N for enabling adj V
+    pbkrtest.limit = 3000,    # limit on N for enabling adj V
+    save.ref.grid = TRUE      # save new ref.grid in .Last.ref.grid
 )
 
 # Utility that returns TRUE if getOption("lsmeans")[[opt]] is TRUE
 .lsm.is.true = function(opt) {
-    x = get.lsm.option(opt, FALSE)
+    x = get.lsm.option(opt)
     if (is.logical(x))  x
     else FALSE
 }
@@ -457,7 +561,25 @@ defaults = list(
 ### Returned ref.grid object has linfct = I and bhat = estimates
 ### Primary reason to do this is with transform = TRUE, then can 
 ### work with linear functions of the transformed predictions
-regrid = function(object, transform = TRUE) {
+regrid = function(object, transform = c("response", "mu", "unlink", "log", "none"), 
+    inv.log.lbl = "response", predict.type) 
+{
+    if (is.logical(transform))   # for backward-compatibility
+        transform = ifelse(transform, "response", "none")
+    else
+        transform = match.arg(transform)
+    
+    # if we have two transformations to undo, do the first one recursively
+    if ((transform == "response") && (!is.null(object@misc$tran2)))
+        object = regrid(object, transform = "mu")
+    
+    # Save post.beta stuff
+    PB = object@post.beta
+    NC = attr(PB, "n.chains")
+    
+    if (!is.na(PB[1])) # fix up post.beta BEFORE we overwrite parameters
+        PB = PB %*% t(object@linfct)
+
     est = .est.se.df(object, do.se = TRUE) ###FALSE)
     estble = !(is.na(est[[1]]))
     object@V = vcov(object)[estble, estble, drop=FALSE]
@@ -467,32 +589,70 @@ regrid = function(object, transform = TRUE) {
         object@nbasis = estimability::all.estble
     else
         object@nbasis = object@linfct[, !estble, drop = FALSE]
-    if(transform && !is.null(object@misc$tran)) {
+    
+    # override the df function
+    df = est$df
+    test.df = diff(range(df))
+    if (is.na(test.df) || test.df < .001) {
+        object@dfargs = list(df = mean(df))
+        object@dffun = function(k, dfargs) dfargs$df
+    }
+    else { # use containment df
+        object@dfargs = list(df = df)
+        object@dffun = function(k, dfargs) {
+            idx = which(zapsmall(k) != 0)
+            ifelse(length(idx) == 0, NA, min(dfargs$df[idx]))
+        }
+    }
+    
+    if(transform %in% c("response", "mu", "unlink", "log") && !is.null(object@misc$tran)) {
         link = attr(est, "link")
         D = .diag(link$mu.eta(object@bhat[estble]))
         object@bhat = link$linkinv(object@bhat)
         object@V = D %*% tcrossprod(object@V, D)
+        if (!is.na(PB[1]))
+            PB = matrix(link$linkinv(PB), ncol = ncol(PB))
         inm = object@misc$inv.lbl
         if (!is.null(inm))
             object@misc$estName = inm
-        object@misc$tran = object@misc$inv.lbl = NULL
-        # override the df function
-        df = est$df
-        if (length(unique(df)) == 1) {
-            object@dfargs = list(df = df[1])
-            object@dffun = function(k, dfargs) dfargs$df
-        }
-        else { # use containment df
-            object@dfargs = list(df = df)
-            object@dffun = function(k, dfargs) {
-                idx = which(zapsmall(k) != 0)
-                ifelse(length(idx) == 0, NA, min(dfargs$df[idx]))
-            }
+        if((transform %in% c("mu", "unlink")) && !is.null(object@misc$tran2)) {
+            object@misc$tran = object@misc$tran2
+            object@misc$tran2 = object@misc$tran.mult = object@misc$inv.lbl = NULL
         }
+        else
+            object@misc$tran = object@misc$tran.mult = object@misc$inv.lbl = NULL
     }
+    if (transform == "log") { # from prev block, we now have stuff on response scale
+        incl = which(object@bhat > 0)
+        if (length(incl) < length(object@bhat)) {
+            message("Non-positive response predictions are flagged as non-estimable")
+            tmp = seq_along(object@bhat)
+            excl = tmp[-incl]
+            object@bhat[excl] = NA
+            object@nbasis = sapply(excl, function(ii) 0 + (tmp == ii))
+        }
+        D = .diag(1/object@bhat[incl])
+        object@V = D %*% tcrossprod(object@V[incl, incl, drop = FALSE], D)
+        object@bhat = log(object@bhat)
+        if (!is.na(PB[1])) {
+            PB[PB <= 0] = NA
+            PB = log(PB)
+            PB[1] = ifelse(is.na(PB[1]), 0, PB[1]) # make sure 1st elt isn't NA
+        }
+        object@misc$tran = "log"
+        object@misc$inv.lbl = inv.log.lbl
+    }
+    
+    if(!is.na(PB[1])) {
+        attr(PB, "n.chains") = NC
+        object@post.beta = PB
+    }
+    
     # Nix out things that are no longer needed or valid
     object@grid$.offset. = object@misc$offset.mult =
         object@misc$estHook = object@misc$vcovHook = NULL
+    if(!missing(predict.type))
+        object = update(object, predict.type = predict.type)
     object
 }
 
